@@ -32,7 +32,6 @@
 
 #include "gdl-i18n.h"
 #include "gdl-switcher.h"
-#include "gdl-tools.h"
 #include "libgdlmarshal.h"
 #include "libgdltypebuiltins.h"
 
@@ -88,7 +87,7 @@ struct _GdlSwitcherPrivate {
     gboolean in_toggle;
 };
 
-GDL_CLASS_BOILERPLATE (GdlSwitcher, gdl_switcher, GtkNotebook, GTK_TYPE_NOTEBOOK)
+G_DEFINE_TYPE (GdlSwitcher, gdl_switcher, GTK_TYPE_NOTEBOOK)
 
 #define INTERNAL_MODE(switcher)  (switcher->priv->switcher_style == \
             GDL_SWITCHER_STYLE_TOOLBAR ? switcher->priv->toolbar_style : \
@@ -253,10 +252,9 @@ button_toggled_callback (GtkToggleButton *toggle_button,
 
 /* Returns -1 if layout didn't happen because a resize request was queued */
 static int
-layout_buttons (GdlSwitcher *switcher)
+layout_buttons (GdlSwitcher *switcher, GtkAllocation* allocation)
 {
     GtkRequisition client_requisition = {0,};
-    GtkAllocation *allocation = & GTK_WIDGET (switcher)->allocation;
     GdlSwitcherStyle switcher_style;
     gboolean icons_only;
     int num_btns = g_slist_length (switcher->priv->buttons);
@@ -274,8 +272,7 @@ layout_buttons (GdlSwitcher *switcher)
     
     last_buttons_height = switcher->priv->buttons_height_request;
     
-    GDL_CALL_PARENT (GTK_WIDGET_CLASS, size_request,
-                     (GTK_WIDGET (switcher), &client_requisition));
+    GTK_WIDGET_CLASS (gdl_switcher_parent_class)->size_request (GTK_WIDGET (switcher), &client_requisition);
 
     y = allocation->y + allocation->height - V_PADDING - 1;
 
@@ -429,14 +426,14 @@ layout_buttons (GdlSwitcher *switcher)
 }
 
 static void
-do_layout (GdlSwitcher *switcher)
+do_layout (GdlSwitcher *switcher, GtkAllocation* allocation)
 {
-    GtkAllocation *allocation = & GTK_WIDGET (switcher)->allocation;
+    
     GtkAllocation child_allocation;
     int y;
-
+    
     if (switcher->priv->show) {
-        y = layout_buttons (switcher);
+        y = layout_buttons (switcher, allocation);
         if (y < 0) /* Layout did not happen and a resize was requested */
             return;
     }
@@ -449,8 +446,7 @@ do_layout (GdlSwitcher *switcher)
     child_allocation.width = allocation->width;
     child_allocation.height = y - allocation->y;
     
-    GDL_CALL_PARENT (GTK_WIDGET_CLASS, size_allocate,
-                     (GTK_WIDGET (switcher), &child_allocation));
+    GTK_WIDGET_CLASS (gdl_switcher_parent_class)->size_allocate (GTK_WIDGET (switcher), &child_allocation);
 }
 
 /* GtkContainer methods.  */
@@ -463,9 +459,9 @@ gdl_switcher_forall (GtkContainer *container, gboolean include_internals,
         GDL_SWITCHER (container);
     GSList *p;
     
-    GDL_CALL_PARENT (GTK_CONTAINER_CLASS, forall,
-                     (GTK_CONTAINER (switcher), include_internals,
-                      callback, callback_data));
+    GTK_CONTAINER_CLASS (gdl_switcher_parent_class)->forall (GTK_CONTAINER (switcher), 
+                                                             include_internals,
+                                                             callback, callback_data);
     if (include_internals) {
         for (p = switcher->priv->buttons; p != NULL; p = p->next) {
             GtkWidget *widget = ((Button *) p->data)->button_widget;
@@ -495,8 +491,7 @@ gdl_switcher_remove (GtkContainer *container, GtkWidget *widget)
             break;
         }
     }
-    GDL_CALL_PARENT (GTK_CONTAINER_CLASS, remove,
-                     (GTK_CONTAINER (switcher), widget));
+    GTK_CONTAINER_CLASS (gdl_switcher_parent_class)->remove (GTK_CONTAINER (switcher), widget);
 }
 
 /* GtkWidget methods.  */
@@ -508,8 +503,7 @@ gdl_switcher_size_request (GtkWidget *widget, GtkRequisition *requisition)
     GSList *p;
     gint button_height = 0;
     
-    GDL_CALL_PARENT (GTK_WIDGET_CLASS, size_request,
-                     (GTK_WIDGET (switcher), requisition));
+    GTK_WIDGET_CLASS (gdl_switcher_parent_class)->size_request (GTK_WIDGET (switcher), requisition);
 
     if (!switcher->priv->show)
         return;
@@ -536,24 +530,23 @@ gdl_switcher_size_request (GtkWidget *widget, GtkRequisition *requisition)
 static void
 gdl_switcher_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 {
-    widget->allocation = *allocation;
-    do_layout (GDL_SWITCHER (widget));
+    do_layout (GDL_SWITCHER (widget), allocation);
+    gtk_widget_set_allocation (widget, allocation);
 }
 
 static gint
-gdl_switcher_expose (GtkWidget *widget, GdkEventExpose *event)
+gdl_switcher_draw (GtkWidget *widget, cairo_t *cr)
 {
     GSList *p;
     GdlSwitcher *switcher = GDL_SWITCHER (widget);
     if (switcher->priv->show) {
         for (p = switcher->priv->buttons; p != NULL; p = p->next) {
             GtkWidget *button = ((Button *) p->data)->button_widget;
-            gtk_container_propagate_expose (GTK_CONTAINER (widget),
-                                            button, event);
+            gtk_container_propagate_draw (GTK_CONTAINER (widget),
+                                          button, cr);
         }
     }
-    return GDL_CALL_PARENT_WITH_DEFAULT (GTK_WIDGET_CLASS, expose_event,
-                                         (widget, event), FALSE);
+    return GTK_WIDGET_CLASS (gdl_switcher_parent_class)->draw (widget, cr);
 }
 
 static void
@@ -568,7 +561,7 @@ gdl_switcher_map (GtkWidget *widget)
             gtk_widget_map (button);
         }
     }
-    GDL_CALL_PARENT (GTK_WIDGET_CLASS, map, (widget));
+    GTK_WIDGET_CLASS (gdl_switcher_parent_class)->map (widget);
 }
 
 /* GObject methods.  */
@@ -628,7 +621,7 @@ gdl_switcher_dispose (GObject *object)
     g_slist_free (priv->buttons);
     priv->buttons = NULL;
 
-    GDL_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
+    G_OBJECT_CLASS (gdl_switcher_parent_class)->dispose (object);
 }
 
 static void
@@ -638,7 +631,7 @@ gdl_switcher_finalize (GObject *object)
 
     g_free (priv);
 
-    GDL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
+    G_OBJECT_CLASS (gdl_switcher_parent_class)->finalize (object);
 }
 
 /* Signal handlers */
@@ -650,14 +643,12 @@ gdl_switcher_notify_cb (GObject *g_object, GParamSpec *pspec,
 }
 
 static void
-gdl_switcher_switch_page_cb (GtkNotebook *nb, GtkNotebookPage *page,
+gdl_switcher_switch_page_cb (GtkNotebook *nb, GtkWidget *page_widget,
                              gint page_num, GdlSwitcher *switcher)
 {
-    GtkWidget       *page_widget;
     gint             switcher_id;
     
     /* Change switcher button */
-    page_widget = gtk_notebook_get_nth_page (nb, page_num);
     switcher_id = gdl_switcher_get_page_id (page_widget);
     gdl_switcher_select_button (GDL_SWITCHER (switcher), switcher_id);
 }
@@ -718,7 +709,7 @@ gdl_switcher_class_init (GdlSwitcherClass *klass)
 
     widget_class->size_request = gdl_switcher_size_request;
     widget_class->size_allocate = gdl_switcher_size_allocate;
-    widget_class->expose_event = gdl_switcher_expose;
+    widget_class->draw = gdl_switcher_draw;
     widget_class->map = gdl_switcher_map;
     
     object_class->dispose  = gdl_switcher_dispose;
@@ -745,11 +736,11 @@ gdl_switcher_class_init (GdlSwitcherClass *klass)
 }
 
 static void
-gdl_switcher_instance_init (GdlSwitcher *switcher)
+gdl_switcher_init (GdlSwitcher *switcher)
 {
     GdlSwitcherPrivate *priv;
 
-    GTK_WIDGET_SET_FLAGS (switcher, GTK_NO_WINDOW);
+    gtk_widget_set_has_window (GTK_WIDGET (switcher), FALSE);
   
     priv = g_new0 (GdlSwitcherPrivate, 1);
     switcher->priv = priv;
